@@ -122,6 +122,14 @@ class GameMap: public sigc::trackable
 	//! figure out where a non-flying unit can't go
         void calculateBlockedAvenues();
 
+	/** Smooth a portion of the map.
+	 *
+	 * Give each tile in the prescribed area the preferred picture for 
+	 * the underlying terrain tile.
+	 */
+	void applyTileStyles (int minx, int miny, int maxx, int maxy,
+			      bool smooth_terrain);
+
     protected:
         //! Create the map with the given tileset
         GameMap(std::string TilesetName);
@@ -137,6 +145,13 @@ class GameMap: public sigc::trackable
         Stack* addArmyAtPos(Vector<int> pos, Army *a);
         bool isBlockedAvenue(int x, int y, int destx, int desty);
         bool isDock(int x, int y);
+	void close_circles (int minx, int miny, int maxx, int maxy);
+
+	TileStyle *calculatePreferredStyle(int i, int j);
+	void demote_lone_tile(int minx, int miny, int minx, int miny, 
+			      Tile::Type intype, Tile::Type outtype);
+	int tile_is_connected_to_other_like_tiles (Tile::Type tile, int i, 
+						   int j);
 
         // Data
         static GameMap* s_instance;
