@@ -26,6 +26,7 @@ struct ShipCacheItem;
 struct CityCacheItem;
 struct TowerCacheItem;
 struct TempleCacheItem;
+struct RuinCacheItem;
 struct StoneCacheItem;
 struct RoadCacheItem;
 struct BridgeCacheItem;
@@ -45,8 +46,8 @@ class City;
   * images has popped up. The player colors are implemented by taking an army
   * picture and a mask, with the mask being a 16 color image, substituting
   * the colors in the mask and blitting the mask over the army (or e.g. city)
-  * image. This takes several blits (>3, there are also things like medals and
-  * level of the unit to be considered) and is therefore costly.
+  * image. This takes several blits (>3, there are also things like medals 
+  * to be considered) and is therefore costly.
   *
   * This class approaches this problem by caching formatted images. You get
   * e.g. an army image by querying the cache, which either gives you the cached
@@ -110,17 +111,19 @@ class GraphicsCache
           * @param armyset      the armyset to be used
           * @param army         the index of the army to be used
           * @param player       the player owning the army
-          * @param level        the level of the unit
           * @return the image of the unit
           */
         SDL_Surface* getArmyPic(Uint32 armyset, Uint32 army, const Player* p,
-                                int level, const bool* medals);
+                                const bool* medals);
 
+        SDL_Surface* getRuinPic(int type);
+        /** Function for getting a ruin picture
+          *
+          * @param type         the type of the ruin
+          * @return image of the ruin 
+          */
         SDL_Surface* getTemplePic(int type);
         /** Function for getting a temple picture
-          *
-          * Most often, we don't need such a sophisticated function. So just
-          * supply the city instance and be happy. :)
           *
           * @param type         the type of the temple
           * @return image of the temple
@@ -149,15 +152,6 @@ class GraphicsCache
           * @param type         the type of the cursor 
           * @return image of the cursor
           */
-        SDL_Surface* getStonePic(int type);
-        /** Function for getting a stone picture
-          *
-          * Most often, we don't need such a sophisticated function. So just
-          * supply the stone type and be happy. :)
-          *
-          * @param type         the type of the stone
-          * @return image of the stone
-          */
 
         SDL_Surface* getShipPic(const Player* p);
         SDL_Surface* getPlantedStandardPic(const Player* p);
@@ -269,8 +263,8 @@ class GraphicsCache
         //! Creates a new temple picture with the given parameters.
         TempleCacheItem* addTemplePic(int type);
 
-        //! Creates a new stone picture with the given parameters.
-        StoneCacheItem* addStonePic(int type);
+        //! Creates a new ruin picture with the given parameters.
+        RuinCacheItem* addRuinPic(int type);
 
         //! Creates a new road picture with the given parameters.
         RoadCacheItem* addRoadPic(int type);
@@ -286,7 +280,7 @@ class GraphicsCache
 
         //! Creates a new army picture with the given parameters.
         ArmyCacheItem* addArmyPic(Uint32 armyset, Uint32 army, const Player* p,
-                                  int level, const bool* medalsbonus);
+                                  const bool* medalsbonus);
 
         //! Creates a new city picture with the given parameters.
         CityCacheItem* addCityPic(int type, const Player* p);
@@ -325,8 +319,8 @@ class GraphicsCache
         //! Erases the oldest (least recently requested) temple cache item.
         void eraseLastTempleItem();
 
-        //! Erases the oldest (least recently requested) stone cache item.
-        void eraseLastStoneItem();
+        //! Erases the oldest (least recently requested) ruin cache item.
+        void eraseLastRuinItem();
 
         //! Erases the oldest (least recently requested) road cache item.
         void eraseLastRoadItem();
@@ -373,17 +367,11 @@ class GraphicsCache
         //! Loads the images for the tower pictures
         void loadTowerPics();
 
-        //! Loads the images for the ship picture and it's mask.
-        void loadShipPic();
-
-        //! Loads the images for the planted standard picture and it's mask.
-        void loadPlantedStandardPic();
-
-        //! Loads the images for the city pictures.
+        //! Loads the images for the temple pictures.
         void loadTemplePics();
 
-        //! Loads the images for the stone pictures.
-        void loadStonePics();
+        //! Loads the images for the ruin pictures.
+        void loadRuinPics();
 
         //! Loads the images for the road pictures.
         void loadRoadPics();
@@ -421,7 +409,7 @@ class GraphicsCache
         std::list<TowerCacheItem*> d_towerlist;
         std::list<FlagCacheItem*> d_flaglist;
         std::list<TempleCacheItem*> d_templelist;
-        std::list<StoneCacheItem*> d_stonelist;
+        std::list<RuinCacheItem*> d_ruinlist;
         std::list<RoadCacheItem*> d_roadlist;
         std::list<FogCacheItem*> d_foglist;
         std::list<BridgeCacheItem*> d_bridgelist;
@@ -434,12 +422,11 @@ class GraphicsCache
         std::list<PlantedStandardCacheItem*> d_plantedstandardlist;
 
         //some private surfaces
-        SDL_Surface* d_levelmask;
         SDL_Surface* d_medalsmask;
         SDL_Surface* d_citypic[MAX_PLAYERS + 1]; //+1 for neutral
         SDL_Surface* d_towerpic[MAX_PLAYERS];
         SDL_Surface* d_templepic[TEMPLE_TYPES];
-        SDL_Surface* d_stonepic[STONE_TYPES];
+        SDL_Surface* d_ruinpic[RUIN_TYPES];
         SDL_Surface* d_roadpic[ROAD_TYPES];
         SDL_Surface* d_bridgepic[BRIDGE_TYPES];
         SDL_Surface* d_cursorpic[CURSOR_TYPES];
@@ -456,10 +443,6 @@ class GraphicsCache
 	SDL_Surface* d_smallruinedcity;
 	SDL_Surface* d_smallhero;
         SDL_Surface* d_movebonuspic[6];
-	SDL_Surface* d_ship;
-	SDL_Surface* d_shipmask;
-	SDL_Surface* d_planted_standard;
-	SDL_Surface* d_planted_standard_mask;
 	SDL_Surface* d_port;
 	SDL_Surface* d_small_ruin_unexplored;
 	SDL_Surface* d_small_stronghold_unexplored;
