@@ -34,16 +34,6 @@ using namespace std;
 QuestCitySack::QuestCitySack (QuestsManager& mgr, Uint32 hero) 
     : Quest(mgr, hero, Quest::CITYSACK)
 {
-    // we want to stay informed about city sacks
-    const Playerlist* pl = Playerlist::getInstance();
-    for (Playerlist::const_iterator it = pl->begin(); it != pl->end(); it++)
-      {
-        (*it)->ssackingCity.connect(
-	    sigc::mem_fun(this, &QuestCitySack::citySacked));
-        (*it)->srazingCity.connect(
-	    sigc::mem_fun(this, &QuestCitySack::cityRazed));
-      }
-
     // find us a victim
     City* c = chooseToSack(getHero()->getPlayer());
     assert(c);      // should never fail because isFeasible is checked first
@@ -58,15 +48,6 @@ QuestCitySack::QuestCitySack (QuestsManager& q_mgr, XML_Helper* helper)
      : Quest(q_mgr, helper)
 {
     // let us stay in touch with the world...
-    const Playerlist* pl = Playerlist::getInstance();
-    for (Playerlist::const_iterator it = pl->begin(); it != pl->end(); it++)
-      {
-        (*it)->ssackingCity.connect(
-	    sigc::mem_fun(this, &QuestCitySack::citySacked));
-        (*it)->srazingCity.connect(
-	    sigc::mem_fun(this, &QuestCitySack::cityRazed));
-      }
-    
     helper->getData(d_city, "city");
     d_targets.push_back(getCity()->getPos());
     initDescription();
@@ -93,7 +74,7 @@ bool QuestCitySack::save(XML_Helper* helper) const
 //=======================================================================
 std::string QuestCitySack::getProgress() const
 {
-    return _("You aren't afraid of doing it, do you?");
+    return _("You aren't afraid of doing it, are you?");
 }
 //=======================================================================
 void QuestCitySack::getSuccessMsg(std::queue<std::string>& msgs) const
@@ -106,7 +87,8 @@ void QuestCitySack::getExpiredMsg(std::queue<std::string>& msgs) const
     char buf[101]; buf[100] = '\0';
     const City* c = getCity();
 
-    snprintf(buf, 100, _("You failed to sack the city \"%s\"."), c->getName().c_str());
+    snprintf(buf, 100, _("The sacking of \"%s\" could not be accomplished."), 
+	     c->getName().c_str());
     msgs.push(buf);
 }
 //=======================================================================
@@ -120,29 +102,6 @@ City* QuestCitySack::getCity() const
     return 0;
 }
 //=======================================================================
-void QuestCitySack::citySacked(City* city, Stack* s, int gold, std::list<Uint32> sacked_types)
-{
-    // some basic considerations have to be done
-    if ((city->getId() != d_city) || !isActive())
-        return;
-    
-    // look if our hero is in the list of sackers
-    for (Stack::const_iterator it = s->begin(); it != s->end(); it++)
-        if ((*it)->isHero() && ((*it)->getId() == d_hero))
-        {
-            debug("CONGRATULATIONS: QUEST 'CITY SACK' IS COMPLETED!");
-            d_q_mgr.questCompleted(d_hero);
-            return;
-        }
-
-}
-void QuestCitySack::cityRazed(City* city, Stack* s)
-{
-    // looks like someone else razed it
-    debug("WHAT A PITY: QUEST 'CITY SACK' CANNOT BE COMPLETED!");
-    d_q_mgr.questExpired(d_hero);
-}
-//=======================================================================
 void QuestCitySack::initDescription()
 {
     const City* c = getCity();
@@ -171,3 +130,22 @@ City * QuestCitySack::chooseToSack(Player *p)
 
     return cities[rand() % cities.size()];
 }
+	 
+void QuestCitySack::armyDied(Army *a, bool heroIsCulprit)
+{
+  ;
+  //this quest does nothing when an army dies
+}
+
+void QuestCitySack::cityAction(City *c, CityDefeatedAction action, 
+			       bool heroIsCulprit, int gold)
+{
+  if (!isActive())
+    return;
+  if (action == CITY_DEFEATED_SACK && heroIsCulprit && c &&
+      c->getId() == d_city)
+    {
+      debug("CONGRATULATIONS: QUEST 'CITY SACK' IS COMPLETED!");
+      d_q_mgr.questCompleted(d_hero);
+    }
+}
