@@ -189,9 +189,10 @@ void QuestsManager::questCompleted(Uint32 heroId)
           }
       }
 
-    debug("deactivate quest");
-    _deactivateQuest(heroId);
-    debug("quest deactivated");
+    //debug("deactivate quest");
+    //_deactivateQuest(heroId);
+    d_quests.erase(heroId);
+    //debug("quest deactivated");
 }
 //========================================================================
 void QuestsManager::questExpired(Uint32 heroId)
@@ -201,7 +202,7 @@ void QuestsManager::questExpired(Uint32 heroId)
     if (quest == 0)
         return;
     
-    quest_expired.emit(quest);
+    //quest_expired.emit(quest);
     
     debug("deactivate quest");
     _deactivateQuest(heroId);
@@ -238,12 +239,16 @@ bool QuestsManager::save(XML_Helper* helper) const
 	bool retval = true;
 	retval &= helper->openTag("questlist");
     
-    std::map<Uint32,Quest*>::const_iterator it;
-    for (it = d_quests.begin(); it != d_quests.end(); it++) 
+    for (std::map<Uint32,Quest*>::const_iterator it = d_quests.begin(); 
+	 it != d_quests.end(); it++) 
       {
-	if (((*it).second)->isPendingDeletion() == false)
-	  retval &= ((*it).second)->save(helper);
+	if ((*it).second == NULL)
+	  continue;
+	retval &= ((*it).second)->save(helper);
       }
+    for (std::list<Quest *>::const_iterator it = d_inactive_quests.begin();
+	 it != d_inactive_quests.end(); it++)
+      retval &= (*it)->save(helper);
 
     debug("Quests saved\n");
 	retval &= helper->closeTag();
@@ -289,7 +294,12 @@ bool QuestsManager::load(string tag, XML_Helper* helper)
         
         debug("quest created: q = " << quest);
         if (quest)
-            d_quests[hero] = quest;
+	  {
+	    if (quest->isPendingDeletion())
+	      d_inactive_quests.push_back(quest);
+	    else
+	      d_quests[hero] = quest;
+	  }
         
         return true;
     }
@@ -299,13 +309,7 @@ bool QuestsManager::load(string tag, XML_Helper* helper)
 //======================================================================
 void QuestsManager::_sharedInit()
 {
-    debug("QuestsManager constructor: connecting to the signals")
-
-    const Playerlist* pl = Playerlist::getInstance();
-    for (Playerlist::const_iterator it = pl->begin(); it != pl->end(); it++)
-        (*it)->sdyingArmy.connect( sigc::mem_fun(*this, &QuestsManager::_dyingArmy));
-
-    sendingTurn.connect( sigc::mem_fun(*this, &QuestsManager::_cleanup));
+    debug("QuestsManager constructor")
 
     // now prepare the vector of pointers to the
     // functions (class static members) checking feasibility
@@ -319,23 +323,11 @@ void QuestsManager::_sharedInit()
     d_questsFeasible.push_back(&(QuestPillageGold::isFeasible));
 }
 //========================================================================
-void QuestsManager::_dyingArmy(Army* army, std::vector<Uint32> culprits)
-{
-    if (!army->isHero())
-        return;
-
-    if (d_quests.count(army->getId()))
-    {
-        debug("QuestsManager: the hero is dead, clearing his quests...");
-        _deactivateQuest(army->getId());
-    }
-}
-//========================================================================
 void QuestsManager::_deactivateQuest(Uint32 heroId)
 {
     Quest *q = d_quests[heroId];
     q->deactivate();
-    d_inactive_quests.push(q);
+    d_inactive_quests.push_back(q);
     // delete it from hash of active quests
     d_quests.erase(heroId);
 }
@@ -344,9 +336,135 @@ void QuestsManager::_cleanup(Player::Type type)
 {
     debug("QuestsManager: _cleanup!");
 
-    while (d_inactive_quests.empty() == false)
+    std::list<Quest *>::iterator it = d_inactive_quests.begin();
+    while(it != d_inactive_quests.end())
+      {
+	Quest *q =(*it);
+	it = d_inactive_quests.erase(it);
+	if (q)
+	  delete q;
+      }
+}
+
+void QuestsManager::armyDied(Army *a, std::vector<Uint32>& culprits)
+{
+  //tell all quests that an army died
+  //each quest takes care of what happens when an army dies
+  std::map<Uint32,Quest*>::iterator it;
+  for (it = d_quests.begin(); it != d_quests.end(); it++) 
     {
-        delete d_inactive_quests.front();
-        d_inactive_quests.pop();
+      if ((*it).second == NULL)
+	continue;
+      if ((*it).second->isPendingDeletion() == true)
+	continue;
+      //was this hero a perpetrator?
+      bool heroIsCulprit = false;
+      for (unsigned int i = 0; i <culprits.size(); i++)
+	{
+	  if (culprits[i] == (*it).second->getHeroId())
+	    {
+	      heroIsCulprit = true;
+	      break;
+	    }
+	}
+      (*it).second->armyDied(a, heroIsCulprit);
+    }
+
+  //is it a hero that has an outstanding quest?
+  //this is what deactivates a quest upon hero death
+  Quest *quest = d_quests[a->getId()];
+  if (quest && quest->isPendingDeletion() == false)
+    questExpired(a->getId());
+
+}
+
+void QuestsManager::cityAction(City *c, Stack *s, 
+			       CityDefeatedAction action, int gold)
+{
+  std::map<Uint32,Quest*>::iterator it;
+  //did any of the heroes who have outstanding quests do this?
+  for (it = d_quests.begin(); it != d_quests.end(); it++) 
+    {
+      if ((*it).second == NULL)
+	continue;
+      if ((*it).second->isPendingDeletion() == true)
+	continue;
+      if (!s)
+	(*it).second->cityAction(c, action, false, gold);
+      else
+	{
+	  for (Stack::iterator sit = s->begin(); sit != s->end(); sit++)
+	    if ((*sit)->getId() == (*it).second->getHeroId())
+	      (*it).second->cityAction(c, action, true, gold);
+	    else
+	      (*it).second->cityAction(c, action, false, gold);
+	}
+    }
+  //is it a city that is part of an outstanding quest?
+  //this is what deactivates a quest upon raze
+  if (action == CITY_DEFEATED_RAZE)
+    {
+      for (it = d_quests.begin(); it != d_quests.end(); it++) 
+	{
+	  bool target_city = false;
+	  if ((*it).second == NULL)
+	    continue;
+	  if ((*it).second->isPendingDeletion() == true)
+	    continue;
+	  if ((*it).second->getType() == Quest::CITYSACK)
+	    {
+	      QuestCitySack *q = dynamic_cast<QuestCitySack *>((*it).second);
+	      if (q->getCityId() == c->getId())
+		target_city = true;
+	    }
+	  if ((*it).second->getType() == Quest::CITYRAZE)
+	    {
+	      QuestCityRaze *q = dynamic_cast<QuestCityRaze *>((*it).second);
+	      if (q->getCityId() == c->getId())
+		target_city = true;
+	    }
+	  if ((*it).second->getType() == Quest::CITYOCCUPY)
+	    {
+	      QuestCityOccupy *q = dynamic_cast<QuestCityOccupy*>((*it).second);
+	      if (q->getCityId() == c->getId())
+		target_city = true;
+	    }
+	  if (target_city)
+	    questExpired((*it).second->getHeroId());
+	}
+    }
+}
+
+void QuestsManager::cityRazed(City *c, Stack *s)
+{
+  cityAction(c, s, CITY_DEFEATED_RAZE, 0);
+  //did we raze a city we care about in another quest?
+}
+void QuestsManager::citySacked(City *c, Stack *s, int gold)
+{
+  cityAction(c, s, CITY_DEFEATED_SACK, gold);
+}
+void QuestsManager::cityPillaged(City *c, Stack *s, int gold)
+{
+  cityAction(c, s, CITY_DEFEATED_PILLAGE, gold);
+}
+void QuestsManager::cityOccupied(City *c, Stack *s)
+{
+  cityAction(c, s, CITY_DEFEATED_OCCUPY, 0);
+}
+void QuestsManager::nextTurn(Player *p)
+{
+  // go through our inactive list and remove quests belonging to us
+  for (std::list<Quest*>::iterator it = d_inactive_quests.begin();
+       it != d_inactive_quests.end(); it++)
+    {
+      if ((*it)->getPlayer() == p)
+	{
+	  quest_expired.emit(*it);
+	  Quest *q = *it;
+	  it = d_inactive_quests.erase(it);
+	  if (q)
+	    delete q;
+	}
     }
 }
