@@ -44,7 +44,8 @@ class NewMapDialog
 	int width, height;
 	int grass, water, swamp, forest, hills, mountains;
 	int cities, ruins, temples;
-	int signposts, stones;
+	int signposts;
+	std::string tileset;
     };
 
     Map map;
@@ -57,6 +58,7 @@ class NewMapDialog
     Gtk::ComboBox *map_size_combobox;
     Gtk::ComboBoxText *fill_style_combobox;
     Gtk::Widget *random_map_container;
+    Gtk::ComboBoxText *tile_theme_combobox;
     Gtk::Scale *grass_scale;
     Gtk::Scale *water_scale;
     Gtk::Scale *swamp_scale;
@@ -67,7 +69,6 @@ class NewMapDialog
     Gtk::Scale *ruins_scale;
     Gtk::Scale *temples_scale;
     Gtk::Scale *signposts_scale;
-    Gtk::Scale *stones_scale;
 
     enum { MAP_SIZE_NORMAL = 0, MAP_SIZE_SMALL, MAP_SIZE_TINY };
 
