@@ -28,6 +28,7 @@
 #include "sound.h"
 #include "GraphicsCache.h"
 #include "GameScenario.h"
+#include "QuestsManager.h"
 #include "NextTurn.h"
 
 #include "gamebigmap.h"
@@ -59,6 +60,56 @@
 
 //#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<flush<<endl;}
 #define debug(x)
+void Game::addPlayer(Player *p)
+{
+
+  //disconnect prior players' connections
+  for (std::list<sigc::connection>::iterator it = 
+       connections[p->getId()].begin(); 
+       it != connections[p->getId()].end(); it++) 
+    (*it).disconnect();
+  connections[p->getId()].clear();
+
+  connections[p->getId()].push_back
+    (p->sdyingStack.connect(sigc::mem_fun(this, &Game::stackDied)));
+
+  if (p->getType() == Player::HUMAN)
+    {
+      connections[p->getId()].push_back
+	(p->snewLevelArmy.connect(sigc::mem_fun(this, &Game::newLevelArmy)));
+      connections[p->getId()].push_back
+	(p->snewMedalArmy.connect(sigc::mem_fun(this, &Game::newMedalArmy)));
+      connections[p->getId()].push_back
+	(p->srecruitingHero.connect
+	 (sigc::bind<0>
+	  (sigc::mem_fun
+	   (hero_offers_service, &sigc::signal<bool, Player *, 
+	    Hero *, City *, int>::emit), p)));
+    }
+	
+      
+  connections[p->getId()].push_back
+    (p->schangingStatus.connect 
+     (sigc::mem_fun(this, &Game::update_sidebar_stats)));
+        
+  connections[p->getId()].push_back
+    (p->supdatingStack.connect (sigc::mem_fun(this, &Game::stackUpdate)));
+  connections[p->getId()].push_back
+    (p->sinvadingCity.connect(sigc::mem_fun(this, &Game::invading_city)));
+  connections[p->getId()].push_back
+    (p->fight_started.connect (sigc::mem_fun(*this, &Game::on_fight_started)));
+  connections[p->getId()].push_back
+    (p->ruinfight_started.connect
+     (sigc::mem_fun
+      (ruinfight_started, &sigc::signal<void, Stack *, Stack *>::emit)));
+  connections[p->getId()].push_back
+    (p->ruinfight_finished.connect
+     (sigc::mem_fun
+      (ruinfight_finished, &sigc::signal<void, Fight::Result>::emit)));
+  connections[p->getId()].push_back
+    (p->advice_asked.connect
+     (sigc::mem_fun(advice_asked, &sigc::signal<void, float>::emit)));
+}
 
 Game::Game(GameScenario* gameScenario)
     : d_gameScenario(gameScenario) 
@@ -85,10 +136,10 @@ Game::Game(GameScenario* gameScenario)
     // init the smallmap
     smallmap.reset(new SmallMap);
     // pass map changes directly through 
+    smallmap->resize();
     smallmap->map_changed.connect(
 	sigc::mem_fun(smallmap_changed,
 		      &sigc::signal<void, SDL_Surface *>::emit));
-    smallmap->resize(GameMap::get_dim() * 2);
 
     // connect the two maps
     bigmap->view_changed.connect(
@@ -104,30 +155,7 @@ Game::Game(GameScenario* gameScenario)
     for (Playerlist::iterator i = pl->begin(); i != pl->end(); ++i)
     {
 	Player *p = *i;
-        p->sdyingStack.connect(sigc::mem_fun(this, &Game::stackDied));
-
-        if (p->getType() == Player::HUMAN)
-        {
-            p->snewLevelArmy.connect(sigc::mem_fun(this, &Game::newLevelArmy));
-            p->snewMedalArmy.connect(sigc::mem_fun(this, &Game::newMedalArmy));
-	    p->srecruitingHero.connect(
-		// bind the player
-		sigc::bind<0>(
-		    sigc::mem_fun(hero_offers_service, &sigc::signal<bool, Player *, Hero *, City *, int>::emit),
-		    p));
-        }
-	
-        p->schangingStatus.connect(
-	    sigc::mem_fun(this, &Game::update_sidebar_stats));
-        p->supdatingStack.connect(sigc::mem_fun(this, &Game::stackUpdate));
-	
-        p->sinvadingCity.connect(sigc::mem_fun(this, &Game::invading_city));
-        p->fight_started.connect(
-	    sigc::mem_fun(*this, &Game::on_fight_started));
-        p->ruinfight_started.connect(
-	    sigc::mem_fun(ruinfight_started, &sigc::signal<void, Stack *, Stack *>::emit));
-        p->ruinfight_finished.connect(
-	    sigc::mem_fun(ruinfight_finished, &sigc::signal<void, Fight::Result>::emit));
+	addPlayer(p);
     }
     pl->splayerDead.connect(sigc::mem_fun(this, &Game::on_player_died));
 
@@ -148,13 +176,20 @@ Game::Game(GameScenario* gameScenario)
 
 Game::~Game()
 {
+  for (unsigned int i = 0; i < MAX_PLAYERS + 1; i++)
+    {
+      for (std::list<sigc::connection>::iterator it = connections[i].begin(); 
+	   it != connections[i].end(); it++) 
+	(*it).disconnect();
+      connections[i].clear();
+    }
     delete d_gameScenario;
     delete d_nextTurn;
 }
 
 void Game::end_turn()
 {
-    bigmap->unselect_active_stack();
+    unselect_active_stack();
     clear_stack_info();
     update_control_panel();
     lock_inputs();
@@ -194,9 +229,10 @@ void Game::update_sidebar_stats()
 	}
 
     s.units = 0;
+    s.upkeep = player->getUpkeep();
     Stacklist *sl = player->getStacklist();
     for (Stacklist::iterator i = sl->begin(), iend = sl->end(); i != iend; ++i)
-	s.units += (*i)->size();
+      s.units += (*i)->size();
     
     s.turns = d_gameScenario->getRound();
     
@@ -220,7 +256,7 @@ void Game::select_next_movable_stack()
   Stacklist *sl = Playerlist::getActiveplayer()->getStacklist();
   Stack* stack = sl->getNextMovable();
   sl->setActivestack(stack);
-  bigmap->select_active_stack();
+  select_active_stack();
 }
 
 void Game::move_selected_stack()
@@ -231,7 +267,7 @@ void Game::move_selected_stack()
   if (stack->canMove() == false)
     {
       Playerlist::getActiveplayer()->getStacklist()->setActivestack(0);
-      bigmap->unselect_active_stack();
+      unselect_active_stack();
     }
 }
 
@@ -243,10 +279,10 @@ void Game::move_all_stacks()
   for (Stacklist::iterator i = sl->begin(), end = sl->end(); i != end; ++i)
     {
       Stack &s = **i;
-      if (!(s.empty()) && !(s.getPath()->empty()) && s.canMove())
+      if (!(s.empty()) && !(s.getPath()->empty()) && s.enoughMoves())
 	{
 	  sl->setActivestack(&s);
-	  bigmap->select_active_stack();
+	  select_active_stack();
 	  player->stackMove(player->getActivestack());
 	  i = sl->begin();
 	}
@@ -255,7 +291,7 @@ void Game::move_all_stacks()
   if (sl->getActivestack()->canMove() == false)
     {
       Playerlist::getActiveplayer()->getStacklist()->setActivestack(0);
-      bigmap->unselect_active_stack();
+      unselect_active_stack();
     }
 }
 
@@ -271,9 +307,9 @@ void Game::defend_selected_stack()
   player->getStacklist()->setActivestack(stack);
 
   if (stack)
-    bigmap->select_active_stack();
+    select_active_stack();
   else
-    bigmap->unselect_active_stack();
+    unselect_active_stack();
 }
 
 void Game::park_selected_stack()
@@ -287,16 +323,23 @@ void Game::park_selected_stack()
   player->getStacklist()->setActivestack(stack);
 
   if (stack)
-    bigmap->select_active_stack();
+    select_active_stack();
   else
-    bigmap->unselect_active_stack();
+    unselect_active_stack();
+}
+
+void Game::deselect_selected_stack()
+{
+  Player *player = Playerlist::getActiveplayer();
+  player->getStacklist()->setActivestack(0);
+  unselect_active_stack();
 }
 
 void Game::center_selected_stack()
 {
   Stack *stack = Playerlist::getActiveplayer()->getActivestack();
   if (stack) 
-    bigmap->select_active_stack();
+    select_active_stack();
 }
 
 void Game::search_selected_stack()
@@ -314,15 +357,29 @@ void Game::search_selected_stack()
       Reward *reward;
 
       reward = player->stackSearchRuin(stack, ruin);
-      if (!reward && ruin->hasSage() == false)
+      if (ruin->hasSage() == true)
+	{
+	  sage_visited.emit(ruin, stack);
+	  reward = ruin->getReward();
+	}
+	  
+      if (reward)
+	{
+	  player->giveReward(Playerlist::getActiveplayer()->getActivestack(),
+			     reward);
+	  redraw();
+	  update_stack_info();
+	  update_control_panel();
+	  ruin_searched.emit(ruin, stack, reward);
+	}
+      else
 	{
 	  redraw();
 	  update_stack_info();
 	  update_control_panel();
-	  return;
 	}
 
-      ruin_searched.emit(ruin, stack, reward);
+
 
       update_sidebar_stats();
     }
@@ -356,7 +413,7 @@ void Game::stackUpdate(Stack* s)
     s = Playerlist::getActiveplayer()->getActivestack();
 
   if (s)
-    bigmap->center_view(s->getPos());
+    smallmap->center_view(s->getPos(), true);
 
   redraw();
 
@@ -367,7 +424,7 @@ void Game::stackUpdate(Stack* s)
 // s is currently unused, but can later be filled with reasonable data
 void Game::stackDied(Stack* s)
 {
-  bigmap->unselect_active_stack();
+  unselect_active_stack();
   redraw();
   update_control_panel();
 }
@@ -588,7 +645,7 @@ void Game::invading_city(City* city)
 	city_visited.emit(city);
     }
 
-  Playerlist::getInstance()->checkPlayers();
+  //fixme: check for end of game here
   redraw();
   update_stack_info();
   update_sidebar_stats();
@@ -620,6 +677,7 @@ void Game::update_control_panel()
       can_center_selected_stack.emit(false);
       can_defend_selected_stack.emit(false);
       can_park_selected_stack.emit(false);
+      can_deselect_selected_stack.emit(false);
       can_search_selected_stack.emit(false);
       can_inspect_selected_stack.emit(false);
       can_plant_standard_selected_stack.emit(false);
@@ -663,6 +721,7 @@ void Game::update_control_panel()
 
   can_defend_selected_stack.emit(stack != 0);
   can_park_selected_stack.emit(stack != 0);
+  can_deselect_selected_stack.emit(stack != 0);
   can_center_selected_stack.emit(stack != 0);
 
   if (stack)
@@ -997,21 +1056,18 @@ bool Game::init_turn_for_player(Player* p)
   // can also have it check for e.g. escape key pressed to interrupt
   // an AI-only game to save/quit.
 
+  center_view_on_city();
   next_player_turn.emit(p, d_gameScenario->getRound() + 1);
   if (p->getType() == Player::HUMAN)
     {
       unlock_inputs();
 
-      Stack* stack = p->getActivestack();
-      if (stack != NULL)
-	bigmap->center_view(stack->getPos());
-      else
-	center_view_on_city();
 
       update_sidebar_stats();
       update_stack_info();
       update_control_panel();
 
+      QuestsManager::getInstance()->nextTurn(p);
 
       maybeRecruitHero(p);
 
@@ -1026,8 +1082,8 @@ bool Game::init_turn_for_player(Player* p)
     }
   else
     {
-      center_view_on_city();
       SDL_Delay(250);
+      QuestsManager::getInstance()->nextTurn(p);
       maybeRecruitHero(p);
       return false;
     }
@@ -1054,13 +1110,15 @@ void Game::center_view_on_city()
 {
   const Player* p = Playerlist::getInstance()->getActiveplayer();
 
+  if (p == Playerlist::getInstance()->getNeutral())
+    return;
   // preferred city is a capital city that belongs to the player 
   for (Citylist::iterator i = Citylist::getInstance()->begin();
        i != Citylist::getInstance()->end(); i++)
     {
       if (i->getPlayer() == p && i->isCapital())
 	{
-	  bigmap->center_view(i->getPos());
+	  smallmap->center_view(i->getPos(), true);
 	  return;
 	}
     }
@@ -1071,8 +1129,18 @@ void Game::center_view_on_city()
     {
       if (i->getPlayer() == p)
 	{
-	  bigmap->center_view(i->getPos());
+	  smallmap->center_view(i->getPos(), true);
 	  break;
 	}
     }
 }
+void Game::select_active_stack()
+{
+  Player *p = Playerlist::getInstance()->getActiveplayer();
+  smallmap->center_view(p->getActivestack()->getPos(), true);
+  bigmap->select_active_stack();
+}
+void Game::unselect_active_stack()
+{
+  bigmap->unselect_active_stack();
+}
