@@ -15,9 +15,11 @@
 #ifndef GAME_H
 #define GAME_H
 
-#include <memory>
 #include <sigc++/signal.h>
 #include <glibmm/ustring.h>
+#include <sigc++/connection.h>
+#include <memory>
+#include <list>
 
 #include "rectangle.h"
 #include "sidebar-stats.h"
@@ -62,6 +64,7 @@ class Game
     void center_selected_stack();
     void defend_selected_stack();
     void park_selected_stack();
+    void deselect_selected_stack();
     void search_selected_stack();
     void move_selected_stack();
     void move_all_stacks();
@@ -85,6 +88,7 @@ class Game
 	can_center_selected_stack,
 	can_defend_selected_stack,
 	can_park_selected_stack,
+	can_deselect_selected_stack,
 	can_search_selected_stack,
 	can_inspect_selected_stack,
 	can_plant_standard_selected_stack,
@@ -99,8 +103,10 @@ class Game
     sigc::signal<void, Glib::ustring, MapTipPosition> map_tip_changed;
     sigc::signal<void, Stack *, MapTipPosition> stack_tip_changed;
     sigc::signal<void, Ruin*, Stack*, Reward*> ruin_searched;
+    sigc::signal<void, Ruin*, Stack*> sage_visited;
     sigc::signal<void, Fight &> fight_started;
     sigc::signal<void, Stack *, Stack *> ruinfight_started;
+    sigc::signal<void, float> advice_asked;
     sigc::signal<void, Fight::Result> ruinfight_finished;
     sigc::signal<bool, Player *, Hero *, City *, int> hero_offers_service;
     sigc::signal<bool, bool, Temple *, int> temple_searched;
@@ -120,6 +126,7 @@ class Game
     sigc::signal<void, Player *> game_over;
     sigc::signal<void, Player *> player_died;
     
+    void addPlayer(Player *p);
 
  private:
 
@@ -174,6 +181,8 @@ class Game
 
     
     void looting_city(City *city, int &gold);
+    void unselect_active_stack();
+    void select_active_stack();
 
     GameScenario* d_gameScenario;
     NextTurn* d_nextTurn;
@@ -185,6 +194,7 @@ class Game
 
     bool input_locked;
 
+    std::list<sigc::connection> connections[MAX_PLAYERS + 1];
 };
 
 #endif
