@@ -117,6 +117,7 @@ bool CityWindow::on_map_mouse_button_event(GdkEventButton *e)
     
     city = prodmap->getCity();
     fill_in_city_info();
+    fill_in_production_toggles();
     fill_in_production_info();
     return true;
 }
@@ -129,7 +130,7 @@ void CityWindow::set_parent_window(Gtk::Window &parent)
 
 void CityWindow::run()
 {
-    prodmap->resize(GameMap::get_dim() * 2);
+    prodmap->resize();
     prodmap->draw();
     dialog->show();
     dialog->run();
@@ -165,7 +166,7 @@ void CityWindow::fill_in_production_toggles()
     GraphicsCache *gc = GraphicsCache::getInstance();
 
     SDL_Surface *s
-	= GraphicsCache::getInstance()->getArmyPic(as, 0, player, 1, NULL);
+	= GraphicsCache::getInstance()->getArmyPic(as, 0, player, NULL);
     Glib::RefPtr<Gdk::Pixbuf> empty_pic
 	= Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, true, 8, s->w, s->h);
     empty_pic->fill(0x00000000);
@@ -180,7 +181,7 @@ void CityWindow::fill_in_production_toggles()
         if (type != -1)
             // use GraphicsCache to load army pics because of player-specific
             // colors
-            pic = to_pixbuf(gc->getArmyPic(as, type, player, 1, NULL));
+            pic = to_pixbuf(gc->getArmyPic(as, type, player, NULL));
 	else
 	    pic = empty_pic;
 	
@@ -236,7 +237,7 @@ void CityWindow::fill_in_production_info()
     GraphicsCache *gc = GraphicsCache::getInstance();
     int slot = city->getProductionIndex();
     SDL_Surface *s
-	= GraphicsCache::getInstance()->getArmyPic(as, 0, player, 1, NULL);
+	= GraphicsCache::getInstance()->getArmyPic(as, 0, player, NULL);
     Glib::RefPtr<Gdk::Pixbuf> empty_pic
 	= Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, true, 8, s->w, s->h);
     empty_pic->fill(0x00000000);
@@ -282,7 +283,7 @@ void CityWindow::fill_in_production_info()
             s3 += String::ucompose(_(", then to %1"), 
                                    dest ? dest->getName() : "Standard");
           }
-      pic = to_pixbuf(gc->getArmyPic(as, a->getType(), player, 1, NULL));
+      pic = to_pixbuf(gc->getArmyPic(as, a->getType(), player, NULL));
     }
     
     current_image->set(pic);
