@@ -172,6 +172,16 @@ void OverviewMap::draw_tile_pixel(Maptile *t, int i, int j)
         break;
     }
 }
+void OverviewMap::resize()
+{
+  /* fixme: these values must align with gui/game-preferences.cpp */
+  if (GameMap::getWidth() <= 50 && GameMap::getWidth() <= 75)
+    resize(GameMap::get_dim() * 4);
+  else if (GameMap::getWidth() <= 70 && GameMap::getWidth() <= 105)
+    resize(GameMap::get_dim() * 3);
+  else
+    resize(GameMap::get_dim() * 2);
+}
 void OverviewMap::resize(Vector<int> max_dimensions)
 {
     if (surface)
