@@ -76,7 +76,7 @@ sigc::signal<void, Player::Type> sendingTurn;
 Player::Player(string name, Uint32 armyset, SDL_Color color, Type type,
 	       int player_no)
     :d_color(color), d_name(name), d_armyset(armyset), d_gold(1000),
-    d_dead(false), d_immortal(false), d_type(type)
+    d_dead(false), d_immortal(false), d_type(type), d_upkeep(0)
 {
     if (player_no != -1)
 	d_id = player_no;
@@ -94,20 +94,21 @@ Player::Player(string name, Uint32 armyset, SDL_Color color, Type type,
     {
       d_fight_order.push_back(i);
     }
+    memset(d_triumph, 0, sizeof(d_triumph));
 }
 
 Player::Player(const Player& player)
     :d_color(player.d_color), d_name(player.d_name), d_armyset(player.d_armyset),
     d_gold(player.d_gold), d_dead(player.d_dead), d_immortal(player.d_immortal),
     d_type(player.d_type), d_id(player.d_id), d_fogmap(player.d_fogmap),
-    d_fight_order(player.d_fight_order)
+    d_fight_order(player.d_fight_order), d_upkeep(player.d_upkeep)
 {
     // as the other player is propably dumped somehow, we need to deep copy
     // everything. This costs a lot, but the only useful situation for this
     // I can think of is a change of the player type, as occurs in the editor.
-    Stacklist::iterator it;
     d_stacklist = new Stacklist();
-    for (it = player.d_stacklist->begin(); it != player.d_stacklist->end(); it++)
+    for (Stacklist::iterator it = player.d_stacklist->begin(); 
+	 it != player.d_stacklist->end(); it++)
     {
         Stack* mine = new Stack(**it);
         // change the stack's loyalty
@@ -122,7 +123,7 @@ Player::Player(const Player& player)
 
     // copy events
     std::list<History*>::const_iterator pit;
-    for (pit = player.d_history.begin(); pit != player.d_history.end(); ait++)
+    for (pit = player.d_history.begin(); pit != player.d_history.end(); pit++)
         d_history.push_back(History::copy(*pit));
 
     // copy fogmap; TBD
@@ -138,6 +139,7 @@ Player::Player(XML_Helper* helper)
     helper->getData(d_dead, "dead");
     helper->getData(d_immortal, "immortal");
     helper->getData(d_type, "type");
+    helper->getData(d_upkeep, "upkeep");
 
     string s;
     helper->getData(s, "color");
@@ -167,6 +169,7 @@ Player::Player(XML_Helper* helper)
     helper->registerTag("history", sigc::mem_fun(this, &Player::load));
     helper->registerTag("stacklist", sigc::mem_fun(this, &Player::load));
     helper->registerTag("fogmap", sigc::mem_fun(this, &Player::load));
+    helper->registerTag("triumphs", sigc::mem_fun(this, &Player::load));
 
 }
 
@@ -398,6 +401,7 @@ bool Player::save(XML_Helper* helper) const
     retval &= helper->saveData("immortal", d_immortal);
     retval &= helper->saveData("type", d_type);
     debug("type of " << d_name << " is " << d_type)
+    retval &= helper->saveData("upkeep", d_upkeep);
 
     std::stringstream fight_order;
     for (std::list<Uint32>::const_iterator it = d_fight_order.begin();
@@ -420,6 +424,35 @@ bool Player::save(XML_Helper* helper) const
     retval &= d_stacklist->save(helper);
     retval &= d_fogmap->save(helper);
 
+    //save the triumphs
+	    
+    helper->openTag("triumphs");
+    for (unsigned int i = 0; i < 5; i++)
+      {
+	std::stringstream tally;
+	for (unsigned int j = 0; j < MAX_PLAYERS; j++)
+	    tally << d_triumph[j][i] << " ";
+	switch (TriumphType(i))
+	  {
+	  case TALLY_HERO:
+	    retval &= helper->saveData("hero", tally.str());
+	    break;
+	  case TALLY_NORMAL:
+	    retval &= helper->saveData("normal", tally.str());
+	    break;
+	  case TALLY_SPECIAL:
+	    retval &= helper->saveData("special", tally.str());
+	    break;
+	  case TALLY_SHIP:
+	    retval &= helper->saveData("ship", tally.str());
+	    break;
+	  case TALLY_FLAG:
+	    retval &= helper->saveData("flag", tally.str());
+	    break;
+	  }
+      }
+    helper->closeTag();
+
     return retval;
 }
 
@@ -468,7 +501,48 @@ bool Player::load(string tag, XML_Helper* helper)
     if (tag == "fogmap")
         d_fogmap = new FogMap(helper);
 
+    if (tag == "triumphs")
+      {
+	for (unsigned int i = 0; i < 5; i++)
+	  {
+	    std::string tally;
+	    std::stringstream stally;
+	    Uint32 val;
+	    switch (TriumphType(i))
+	      {
+	      case TALLY_HERO:
+		helper->getData(tally, "hero");
+		break;
+	      case TALLY_NORMAL:
+		helper->getData(tally, "normal");
+		break;
+	      case TALLY_SPECIAL:
+		helper->getData(tally, "special");
+		break;
+	      case TALLY_SHIP:
+		helper->getData(tally, "ship");
+		break;
+	      case TALLY_FLAG:
+		helper->getData(tally, "flag");
+		break;
+	      }
+	    stally.str(tally);
+	    for (unsigned int j = 0; j < MAX_PLAYERS; j++)
+	      {
+		stally >> val;
+		d_triumph[j][i] = val;
+	      }
+	  }
+      }
+
     return true;
 }
 
+void Player::calculateUpkeep()
+{
+    d_upkeep = 0;
+    Stacklist *sl = getStacklist();
+    for (Stacklist::iterator i = sl->begin(), iend = sl->end(); i != iend; ++i)
+      d_upkeep += (*i)->getUpkeep();
+}
 // End of file
